from __future__ import with_statement
from etsy._core import API, MethodTableCache, missing
from cgi import parse_qs
from urlparse import urlparse
import os
from util import Test
import tempfile

    

class MockAPI(API):
    api_url = 'http://host'
    api_version = 'v1'


    def etsy_home(self):
        return Test.scratch_dir


    def get_method_table(self, *args):
        return [{'name': 'testMethod', 
                 'uri': '/test/{test_id}', 
                 'http_method': 'GET', 
                 'params': { 
                     'limit': 'int',
                     'test_id': 'user_id_or_name',
                     'offset': 'int',
                     'fizz': 'enum(foo, bar, baz)',
                     'buzz': 'float',
                     'blah': 'unknown type',
                     'kind': 'string',
                     },
                 'type': 'int', 
                 'description': 'test method.'},
                {'name': 'noPositionals',
                 'uri': '/blah',
                 'http_method': 'GET',
                 'params': {'foo': 'int'},
                 'type': 'int',
                 'description': 'no pos arguments'}]


    def _get_url(self, url, http_method, content_type, body):
        return '{ "count": 1, "results": [3] }'



class MockLog(object):
    def __init__(self, test):
        self.lines = []
        self.test = test

    def __call__(self, msg):
        self.lines.append(msg)


    def assertLine(self, msg):
        failmsg = 'Could not find "%s" in the log. The log was:\n\n%s' % (
            msg, '\n'.join(['   %s' % x for x in self.lines]))
        self.test.assertTrue(msg in self.lines, failmsg)



class CoreTests(Test):
    def setUp(self):
        self.api = MockAPI('apikey', log=MockLog(self))


    def last_query(self):
        qs = urlparse(self.api.last_url).query
        return parse_qs(qs)


    def test_method_created(self):
        self.assertTrue('testMethod' in dir(self.api))


    def test_url_params(self):
        self.api.testMethod(test_id='foo')
        self.assertEqual(self.api.last_url, 
                         'http://host/test/foo?api_key=apikey')


    def test_count_saved(self):
        self.api.testMethod(test_id='foo')
        self.assertTrue(self.api.count)


    def test_results_returned(self):
        x = self.api.testMethod(test_id='foo')
        self.assertEquals(x, [3])


    def test_query_params(self):
        self.api.testMethod(test_id='foo', limit=1)
        self.assertEqual(self.last_query(), {
                'api_key': ['apikey'],
                'limit': ['1'],
                })


    def test_docstring_set(self):
        self.assertEquals(self.api.testMethod.__doc__,
                          'test method.')



    def test_api_url_required(self):
        msg = self.assertRaises(AssertionError, API, '')
        self.assertEqual('No api_url configured.', msg)


    def test_api_url_cannot_end_with_slash(self):
        class Foo(API):
            api_url = 'http://host/'

        msg = self.assertRaises(AssertionError, Foo, '')
        self.assertEqual('api_url should not end with a slash.', msg)


    def test_api_should_define_version(self):
        class Foo(API):
            api_url = 'http://host'

        msg = self.assertRaises(AssertionError, Foo)
        self.assertEqual(msg, 'API object should define api_version')


    def test_key_file_does_not_exist(self):
        msg = self.assertRaises(AssertionError, MockAPI, 
                                key_file='this does not exist')
        self.assertTrue("'this does not exist' does not exist" in msg)


    def test_reading_api_key(self):
        with open('testkeys', 'w') as f:
            f.write("v1 = 'abcdef'")
        try:
            self.assertEqual(MockAPI(key_file='testkeys').api_key, 'abcdef')
        finally:
            os.unlink('testkeys')


    def test_unrecognized_kwarg(self):
        msg = self.assertRaises(ValueError, self.api.testMethod, 
                                test_id=1, not_an_arg=1)
        self.assertEqual(msg, 'Unexpected argument: not_an_arg=1')

    
    def test_unknown_parameter_type_is_passed(self):
        self.api.testMethod(test_id=1, blah=1)
        self.assertEqual(self.last_query()['blah'], ['1'])


    def test_parameter_type_int(self):
        self.api.testMethod(test_id=1, limit=5)
        self.assertEqual(self.last_query()['limit'], ['5'])


    def test_parameter_type_long(self):
        self.api.testMethod(test_id=1L, limit=5L)
        self.assertEqual(self.last_query()['limit'], ['5'])


    def bad_value_msg(self, name, t, v):
        return "Bad value for parameter %s of type '%s' - %s" % (name, t, v)

    def test_invalid_parameter_type_int(self):
        msg = self.assertRaises(ValueError, self.api.testMethod, 
                                test_id=1, limit=5.6)
        self.assertEqual(msg, self.bad_value_msg('limit', 'int', 5.6))


    def test_parameter_type_float(self):
        self.api.testMethod(test_id=1, buzz=42.1)
        self.assertEqual(self.last_query()['buzz'], ['42.1'])


    def test_invalid_parameter_type_float(self):
        msg = self.assertRaises(ValueError, self.api.testMethod, 
                                test_id=1, buzz='x')
        self.assertEqual(msg, self.bad_value_msg('buzz', 'float', 'x'))

    
    def test_int_accepted_as_float(self):
        self.api.testMethod(test_id=1, buzz=3)
        self.assertEqual(self.last_query()['buzz'], ['3'])

        
    def test_parameter_type_enum(self):
        self.api.testMethod(test_id=1, fizz='bar')
        self.assertEqual(self.last_query()['fizz'], ['bar'])


    def test_invalid_parameter_type_enum(self):
        msg = self.assertRaises(ValueError, self.api.testMethod, 
                                test_id=1, fizz='goo')
        self.assertEqual(msg, self.bad_value_msg(
                'fizz', 'enum(foo, bar, baz)', 'goo'))


    def test_parameter_type_string(self):
        self.api.testMethod(test_id=1, kind='blah')
        self.assertEqual(self.last_query()['kind'], ['blah'])


    def test_invalid_parameter_type_string(self):
        msg = self.assertRaises(ValueError, self.api.testMethod, 
                                test_id=1, kind=Test)
        self.assertEqual(msg, self.bad_value_msg('kind', 'string', Test))


    def test_url_arguments_work_positionally(self):
        self.api.testMethod('foo')
        self.assertEqual(self.api.last_url, 
                         'http://host/test/foo?api_key=apikey')


    def test_method_with_no_positionals_doesnt_accept_them(self):
        msg = self.assertRaises(ValueError, self.api.noPositionals, 1, 2)
        self.assertEqual('Positional argument(s): (1, 2) provided, but this '
                         'method does not support them.', msg)

    
    def test_too_many_positionals(self):
        msg = self.assertRaises(ValueError, self.api.testMethod, 1, 2)
        self.assertEqual('Too many positional arguments.', msg)


    def test_positional_argument_not_provided(self):
        msg = self.assertRaises(ValueError, self.api.testMethod)
        self.assertEqual("Required argument 'test_id' not provided.", msg)


    def test_positional_argument_duplicated_in_kwargs(self):
        msg = self.assertRaises(ValueError, self.api.testMethod, 1, test_id=2)
        self.assertEqual('Positional argument duplicated in kwargs: test_id', 
                         msg)


    def test_api_key_and_key_file_both_passed(self):
        msg = self.assertRaises(AssertionError, MockAPI, 
                                api_key='x', key_file='y')
        self.assertEqual('Keys can be read from a file or passed, but not both.',
                         msg)


    def test_logging_works(self):
        self.api.log('foo')
        self.api.log.assertLine('foo')


    def test_log_at_startup(self):
        self.api.log.assertLine('Creating v1 Etsy API, base url=http://host.')






class MockAPI_NoMethods(MockAPI):
    def _get_methods(self, method_cache):
        pass



class MethodTableCacheTests(Test):


    def cache(self, method_cache=missing):
        self.api = MockAPI_NoMethods('apikey')
        self._cache = MethodTableCache(self.api, method_cache)
        return self._cache


    def test_uses_etsy_home_if_exists(self):
        c = self.cache()
        self.assertEqual(os.path.dirname(c.filename), self.scratch_dir)


    def test_uses_temp_dir_if_no_etsy_home(self):
        self.delete_scratch()
        c = self.cache()
        self.assertEqual(os.path.dirname(c.filename), tempfile.gettempdir())


    def test_uses_provided_file(self):
        fn = os.path.join(self.scratch_dir, 'foo.json')
        self.assertEqual(self.cache(method_cache=fn).filename, fn)


    def test_multiple_versions(self):
        c = self.cache()

        class MockAPI2(MockAPI):
            api_version = 'v3'

        self.assertNotEqual(MockAPI2('key').method_cache.filename, c.filename)


    def get_uncached(self):
        c = self.cache()
        return c.get()
    

    def test_no_cache_file_returns_results(self):
        self.assertEqual(2, len(self.get_uncached()))


    def test_no_cache_file_writes_cache(self):
        self.get_uncached()
        self.assertTrue(self._cache.wrote_cache)

    
    def test_no_cache_file(self):
        self.get_uncached()
        self.assertFalse(self._cache.used_cache)


    def get_cached(self):
        c = self.cache()
        c.get()
        c = self.cache()
        return c.get()


    def test_caching(self):
        self.get_cached()
        self.assertTrue(self._cache.used_cache)


    def test_caching_returns_results(self):
        self.assertEqual(2, len(self.get_cached()))


    def test_caching_doesnt_overwrite_cache(self):
        self.get_cached()
        self.assertFalse(self._cache.wrote_cache)


    def make_old_cache(self):
        self.get_cached()
        fn = self._cache.filename
        s = os.stat(fn)
        os.utime(fn, (s.st_atime, s.st_mtime - 48*60*60))


    def test_expired(self):
        self.make_old_cache()
        c = self.cache()
        c.get()
        self.assertFalse(c.used_cache)
        
    
    def test_none_passed_does_not_cache(self):
        self.get_cached()
        c = self.cache(method_cache=None)
        c.get()
        self.assertFalse(c.used_cache)


    def log_tester(self, method_cache=missing):
        return MockAPI('key', method_cache=method_cache, log=MockLog(self))


    def test_logs_when_not_using_cache(self):
        api = self.log_tester(None)
        api.log.assertLine('Not using cached method table.')


    def test_logs_when_method_table_too_old(self):
        self.make_old_cache()
        self.log_tester().log.assertLine('Method table too old.')

    
    def test_logs_when_reading_cache(self):
        api = MockAPI('key')
        self.log_tester().log.assertLine('Reading method table cache: %s' % 
                                         api.method_cache.filename)


    def test_logs_when_not_writing_new_cache(self):
        api = self.log_tester(None)
        api.log.assertLine(
            'Method table caching disabled, not writing new cache.')


    def test_logs_when_writing_new_cache(self):
        t = self.log_tester()
        t.log.assertLine('Wrote method table cache: %s' % 
                         t.method_cache.filename)




#!/usr/bin/env python

import os, sys
import oauth2 as oauth
import webbrowser
from etsy import Etsy, EtsyEnvProduction
from etsy.oauth import EtsyOAuthClient

logging_enabled = True
etsy_env = EtsyEnvProduction()

def my_log(msg):
    if logging_enabled: print(msg)

def write_config_file(oauth_token):
    os.umask(0077)
    config_file = file('config.py', 'w')

    if config:
        config_file.write("oauth_consumer_key = %r\n" % config.oauth_consumer_key)
        config_file.write("oauth_consumer_secret = %r\n" % config.oauth_consumer_secret)

    if oauth_token:
        config_file.write("oauth_token_key = %r\n" % oauth_token.key)
        config_file.write("oauth_token_secret = %r\n" % oauth_token.secret)

try:
    import config
except ImportError:
    config = None
    write_config_file(oauth_token=None)

if hasattr(config, 'oauth_consumer_key') and hasattr(config, 'oauth_consumer_secret'):
    oauth_client = EtsyOAuthClient(
        oauth_consumer_key=config.oauth_consumer_key,
        oauth_consumer_secret=config.oauth_consumer_secret,
        etsy_env=etsy_env)
else:
    sys.stderr.write('ERROR: You must set oauth_consumer_key and oauth_consumer_secret in config.py\n')
    sys.exit(1)

if hasattr(config, 'oauth_token_key') and hasattr(config, 'oauth_token_secret'):
    oauth_client.token = oauth.Token(
        key=config.oauth_token_key,
        secret=config.oauth_token_secret)
else:
    webbrowser.open(oauth_client.get_signin_url())
    oauth_client.set_oauth_verifier(raw_input('Enter OAuth verifier: '))
    write_config_file(oauth_client.token)

etsy_api = Etsy(etsy_oauth_client=oauth_client, etsy_env=etsy_env, log=my_log)

# print 'oauth access token: (key=%r; secret=%r)' % (oauth_client.token.key, oauth_client.token.secret)

print('findAllShopListingsActive => %r' % etsy_api.findAllShopListingsActive(shop_id=config.user_id, sort_on='created', limit=1))

# print('getListing => %r' % etsy_api.getListing(listing_id=63067548))

print('findAllUserShippingTemplates => %r' % etsy_api.findAllUserShippingTemplates(user_id=config.user_id))

def testCreateListing():
    print "Creating listing..."
    
    result = etsy_api.createListing(
        description=config.description,
        title=config.title,
        price=config.price,
        tags=config.tags,
        materials=config.materials,
        shipping_template_id=config.shipping_template_id,
        shop_section_id=config.shop_section_id,
        quantity=config.quantity)

    listing_id = result[0]['listing_id']

    print "Created listing with listing id %d" % listing_id

    result = etsy_api.uploadListingImage(listing_id=listing_id, image=config.image_file)

    print "Result of uploading image: %r" % result

testCreateListing()




import urllib
from etsy._core import API, missing
from etsy.etsy_env import EtsyEnvProduction

class EtsyV2(API):
    api_version = 'v2'

    def __init__(self, api_key='', key_file=None, method_cache=missing, 
                 etsy_env=EtsyEnvProduction(), log=None, etsy_oauth_client=None):
        self.api_url = etsy_env.api_url
        self.etsy_oauth_client = None

        if etsy_oauth_client:
            self.etsy_oauth_client = etsy_oauth_client
            # including api_key in requests when using oauth causes etsy to return 403 Forbidden
            api_key = None
            key_file = None

        super(EtsyV2, self).__init__(api_key, key_file, method_cache, log)

    def _get_url(self, url, http_method, body):
        if self.etsy_oauth_client is not None:
            return self.etsy_oauth_client.do_oauth_request(url, http_method, body)
        return API._get_url(self, url, http_method, body)



from __future__ import with_statement
import json
from urllib.parse import urlencode
import os
import re
import tempfile
import mimetypes
import time
import requests


missing = object()




class TypeChecker(object):
    def __init__(self):
        self.checkers = {
            'int': self.check_int,
            'float': self.check_float,
            'string': self.check_string,
            }


    def __call__(self, method, **kwargs):
        params = method['params']
        for k, v in kwargs.items():
            if k == 'includes': continue

            if k not in params:
                raise ValueError('Unexpected argument: %s=%s' % (k, v))
            
            t = params[k]
            checker = self.checkers.get(t, None) or self.compile(t)
            ok, converted = checker(v)
            if not ok:
                raise ValueError(
                    "Bad value for parameter %s of type '%s' - %s" % (k, t, v))
            kwargs[k] = converted


    def compile(self, t):
        if t.startswith('enum'):
            f = self.compile_enum(t)
        else:
            f = self.always_ok
        self.checkers[t] = f
        return f


    def compile_enum(self, t):
        terms = [x.strip() for x in t[5:-1].split(',')]
        def check_enum(value):
            return (value in terms), value
        return check_enum


    def always_ok(self, value):
        return True, value


    def check_int(self, value):
        return isinstance(value, int), value

    
    def check_float(self, value):
        if isinstance(value, int):
            return True, value
        return isinstance(value, float), value

    
    def check_string(self, value):
        return isinstance(value, str), value




class APIMethod(object):
    def __init__(self, api, spec):
        """
        Parameters:
            api          - API object that this method is associated with.
            spec         - dict with the method specification; e.g.:

              {'name': 'createListing', 'uri': '/listings', 'visibility':
              'private', 'http_method': 'POST', 'params': {'description':
              'text', 'tags': 'array(string)', 'price': 'float', 'title':
              'string', 'materials': 'array(string)', 'shipping_template_id':
              'int', 'quantity': 'int', 'shop_section_id': 'int'}, 'defaults':
              {'materials': None, 'shop_section_id': None}, 'type': 'Listing',
              'description': 'Creates a new Listing'} 
        """

        self.api = api
        self.spec = spec
        self.type_checker = self.api.type_checker
        self.__doc__ = self.spec['description']
        self.compiled = False
    

    def __call__(self, *args, **kwargs):
        if not self.compiled:
            self.compile()
        return self.invoke(*args, **kwargs)


    def compile(self):
        uri = self.spec['uri']
        self.positionals = re.findall('{(.*)}', uri)

        for p in self.positionals:
            uri = uri.replace('{%s}' % p, '%%(%s)s' % p)
        self.uri_format = uri

        self.compiled = True


    def invoke(self, *args, **kwargs):
        if args and not self.positionals:
            raise ValueError(
                'Positional argument(s): %s provided, but this method does '
                'not support them.' % (args,))

        if len(args) > len(self.positionals):
            raise ValueError('Too many positional arguments.')

        for k, v in zip(self.positionals, args):
            if k in kwargs:
                raise ValueError(
                    'Positional argument duplicated in kwargs: %s' % k)
            kwargs[k] = v

        ps = {}
        for p in self.positionals:
            if p not in kwargs:
                raise ValueError("Required argument '%s' not provided." % p)
            ps[p] = kwargs[p]
            del kwargs[p]

        self.type_checker(self.spec, **kwargs)
        return self.api._get(self.spec['http_method'], self.uri_format % ps, **kwargs)




class MethodTableCache(object):
    max_age = 60*60*24

    def __init__(self, api, method_cache):
        self.api = api
        self.filename = self.resolve_file(method_cache)
        self.used_cache = False
        self.wrote_cache = False


    def resolve_file(self, method_cache):
        if method_cache is missing:
            return self.default_file()
        return method_cache


    def etsy_home(self):
        return self.api.etsy_home()


    def default_file(self):
        etsy_home = self.etsy_home()
        d = etsy_home if os.path.isdir(etsy_home) else tempfile.gettempdir()
        return os.path.join(d, 'methods.%s.json' % self.api.api_version)


    def get(self):
        ms = self.get_cached()
        if not ms:
            ms = self.api.get_method_table()
            self.cache(ms)
        return ms
        

    def get_cached(self):
        if self.filename is None or not os.path.isfile(self.filename):
            self.api.log('Not using cached method table.')
            return None
        if time.time() - os.stat(self.filename).st_mtime > self.max_age:
            self.api.log('Method table too old.')
            return None
        with open(self.filename, 'r') as f:
            self.used_cache = True
            self.api.log('Reading method table cache: %s' % self.filename)
            return json.loads(f.read())


    def cache(self, methods):
        if self.filename is None:
            self.api.log('Method table caching disabled, not writing new cache.')
            return
        with open(self.filename, 'w') as f:
            json.dump(methods, f)
            self.wrote_cache = True
            self.api.log('Wrote method table cache: %s' % self.filename)




class API(object):
    def __init__(self, api_key='', key_file=None, method_cache=missing, 
                 log=None):
        """
        Creates a new API instance. When called with no arguments, 
        reads the appropriate API key from the default ($HOME/.etsy/keys) 
        file. 

        Parameters:
            api_key      - An explicit API key to use.
            key_file     - A file to read the API keys from. 
            method_cache - A file to save the API method table in for 
                           24 hours. This speeds up the creation of API
                           objects. 
            log          - An callable that accepts a string parameter.
                           Receives log messages. No logging is done if
                           this is None.

        Only one of api_key and key_file may be passed.

        If method_cache is explicitly set to None, no method table
        caching is performed. If the parameter is not passed, a file in 
        $HOME/.etsy is used if that directory exists. Otherwise, a 
        temp file is used. 
        """
        if not getattr(self, 'api_url', None):
            raise AssertionError('No api_url configured.')

        if self.api_url.endswith('/'):
            raise AssertionError('api_url should not end with a slash.')

        if not getattr(self, 'api_version', None):
            raise AssertionError('API object should define api_version')

        if api_key and key_file:
            raise AssertionError('Keys can be read from a file or passed, '
                                 'but not both.')

        if api_key:
            self.api_key = api_key
        elif key_file:
            self.api_key = self._read_key(key_file)

        self.log = log or self._ignore
        if not callable(self.log):
            raise ValueError('log must be a callable.')

        self.type_checker = TypeChecker()

        self.decode = json.loads

        self.log('Creating %s Etsy API, base url=%s.' % (
                self.api_version, self.api_url))
        self._get_methods(method_cache)



    def _ignore(self, _):
        pass


    def _get_methods(self, method_cache):
        self.method_cache = MethodTableCache(self, method_cache)
        ms = self.method_cache.get()
        self._methods = dict([(m['name'], m) for m in ms])

        for method in ms:
            setattr(self, method['name'], APIMethod(self, method))

        # self.log('API._get_methods: self._methods = %r' % self._methods)


    def etsy_home(self):
        return os.path.expanduser('~/.etsy')


    def get_method_table(self):
        return self._get('GET', '/')


    def _read_key(self, key_file):
        key_file = key_file or os.path.join(self.etsy_home(), 'keys')
        # api_key isn't used when using the oauth client
        # if not os.path.isfile(key_file):
        #     raise AssertionError(
        #         "The key file '%s' does not exist. Create a key file or "
        #         'pass an API key explicitly.' % key_file)

        gs = {}
        execfile(key_file, gs)
        return gs[self.api_version]
        

    def _get_url(self, url, http_method, data):
        self.log("API._get_url: url = %r" % url)
        return requests.request(http_method, url, data=data)
  
    def _get(self, http_method, url, **kwargs):
        if hasattr(self, 'api_key'):
            kwargs.update(dict(api_key=self.api_key))

        data = None
        if http_method == 'GET' or http_method == 'DELETE':
            url = '%s%s?%s' % (self.api_url, url, urlencode(kwargs))
        elif http_method == 'POST' or http_method == 'PUT':
            url = '%s%s' % (self.api_url, url)

            data = {}
            for name, value in kwargs.items():
                if hasattr(value, 'read'):
                    file_mimetype = mimetypes.guess_type(value.name) or 'application/octet-stream'
                    data[name] = (value.name, value.read(), file_mimetype)
                else:
                    data[name] = (None, str(value))

        self.last_url = url
        response = self._get_url(url, http_method, data)

        self.log('API._get: http_method = %r, url = %r, data = %r' % (http_method, url, data))

        try:
            self.data = self.decode(response.text)
        except json.JSONDecodeError:
            raise ValueError('Could not decode response from Etsy as JSON: status_code: %r, text: %r, url %r' \
                % (response.status_code, response.text, response.url))

        self.count = self.data['count']
        return self.data['results']



from ._v2 import EtsyV2 as Etsy
from .etsy_env import EtsyEnvProduction


__version__ = '0.4.0'
__author__ = 'Sean Scheetz'
__copyright__ = 'Copyright 2010, Etsy Inc.'
__license__ = 'GPL v3'
__email__ = 'file_an_issue_on_github@gmail.com'




class EtsyEnvProduction(object):
    request_token_url = 'https://openapi.etsy.com/v2/oauth/request_token'
    access_token_url = 'https://openapi.etsy.com/v2/oauth/access_token'
    signin_url = 'https://www.etsy.com/oauth/signin'
    api_url = 'https://openapi.etsy.com/v2'



import requests
from urllib.parse import quote
from requests_oauthlib import OAuth1Session
from etsy.etsy_env import EtsyEnvProduction

# TODO add support for generating the oauth credentials - may want to inherit from OAuth1Session
class EtsyOAuthClient():
    '''
    You must perform the oauth authentication on your own. This client
    expects valid oauth crendentials for the requesting user.

    client_key is keystring for the etsy app.
    client_secret is the shared secret for the etsy app.
    resource_owner_key is the oauth_token for the user whose data is being retrieved.
    resource_owner_secret is the oauth_token_secret for the user whose data is being retrieved.
    '''
    def __init__(self, client_key, client_secret, resource_owner_key, resource_owner_secret, logger=None):
        self.oauth1Session = OAuth1Session(client_key,
                                           client_secret=client_secret,
                                           resource_owner_key=resource_owner_key,
                                           resource_owner_secret=resource_owner_secret)
        self.logger = logger

    def do_oauth_request(self, url, http_method, data):
        # TODO data seems to work for PUT and POST /listing. See if data 
        # can handle image/actual file data updates if so don't need to split path.
        if (http_method == "POST"):
            response = self.oauth1Session.request(http_method, url, files=data)
        else:
            response = self.oauth1Session.request(http_method, url, data=data)

        if self.logger:
            self.logger.debug('do_oauth_request: response = %r' % response)

        return response

class EtsyOAuthHelper:
    '''
    Used to get the oauth token for the user you want to make requests with.
    TODO there might be a good way to make a class out of this, but because the
    methods are called in separate http requests the state would be lost in between
    calls.
    '''
    @staticmethod
    def get_request_url_and_token_secret(api_key, shared_secret, permission_scopes=[], callback_uri=None, etsy_env=EtsyEnvProduction()):
        '''
        This method implements the first step of the Oauth1.0 3-legged work flow.
        
        Returns the  the login_url for the user you wish to authenticate with your app
        and the oauth_token_secret which must be passed to the get_oauth_token method
        (the next step in the oauth_workflow). You probably want to redirect the user
        to the login_url after this step.

        api_key is the keystring from etsy
        shared_secret is the shared secret from etsy
        permission_scopes is a list of strings. one string per requested permission scope. See link below.
            https://www.etsy.com/developers/documentation/getting_started/oauth#section_permission_scopes
        callback_uri is a path in your application where the user should be redirected after login
        etsy_env is always prod because there is only one etsy environment as of now
        '''
        oauth = OAuth1Session(api_key, client_secret=shared_secret, callback_uri=callback_uri)

        permissions = ' '.join(permission_scopes)
        request_token_url = etsy_env.request_token_url + '?scope=' + quote(permissions)
        request_token_response = oauth.fetch_request_token(request_token_url)
        
        login_url = request_token_response['login_url']
        temp_oauth_token_secret = request_token_response['oauth_token_secret']

        return (login_url, temp_oauth_token_secret)

    @staticmethod
    def get_oauth_token(api_key, shared_secret, oauth_token_secret, auth_url, etsy_env=EtsyEnvProduction()):
        '''
        Retrieves the oauth_token and oauth_token_secret for the user. These are 
        used along with the api_key, shared_secret, and oauth_token_secret (which
        is returned by get_request_url_and_token_secret) to create an EtsyOAuthClient.
        
        api_key is the keystring from etsy
        shared_secret is the shared secret from etsy
        oauth_token_secret is the token_secret returned from get_request_url_and_token_secret
        auth_url is the url used to start the current request. It is the callback_uri
            from get_request_url_and_token_secret with the extra parameters etsy appended
            to it after the user authenticated themselves.
        etsy_env is always prod because there is only one etsy environment as of now
        '''
        oauth = OAuth1Session(api_key, shared_secret)
        oauth_response = oauth.parse_authorization_response(auth_url)
        oauth = OAuth1Session(api_key,
                  client_secret=shared_secret,
                  resource_owner_key=oauth_response['oauth_token'],
                  resource_owner_secret=oauth_token_secret,
                  verifier=oauth_response['oauth_verifier'])
        
        oauth_tokens = oauth.fetch_access_token(etsy_env.access_token_url)
        oauth_token = oauth_tokens['oauth_token']
        oauth_token_secret = oauth_tokens['oauth_token_secret']

        return (oauth_token, oauth_token_secret)



#!/usr/bin/python
import os
from setuptools import setup

this_dir = os.path.realpath(os.path.dirname(__file__))
long_description = open(os.path.join(this_dir, 'README.md'), 'r').read()

setup(
    name = 'etsy2',
    version = '0.4.0',
    author = 'Sean Scheetz',
    author_email = 'contact_through_github@gmail.com',
    description = 'Python access to the Etsy API',
    license = 'GPL v3',
    keywords = 'etsy api handmade',
    packages = ['etsy'],
    long_description = long_description,
    long_description_content_type="text/markdown",
    test_suite = 'test',
    project_urls = {
        'Source Code': 'https://github.com/sscheetz/etsy-python2'
    },
    install_requires=['requests_oauthlib'],
)